#ifndef WIFI_INTERFACE_H_
#define WIFI_INTERFACE_H_
#include "os_type.h"

typedef enum {
	WIFI_TIMEOUT_FAILURE = 0,
	WIFI_NO_STORED_APS_FAILURE,
	WIFI_NO_AVAILABLE_APS_FAILURE,
	WIFI_AP_DISCONNECTED,
	WIFI_AP_AUTHMODE_CHANGE,
	WIFI_SCAN_FAILURE
} wifi_failure_et;

typedef void (* wifi_fail_function_t)(wifi_failure_et);
typedef void (* wifi_success_function_t)(void);

typedef struct {
	wifi_success_function_t 	success_cb;
	wifi_fail_function_t		fail_cb;
	os_timer_t 					timeout_timer;
	uint32_t 					timeout_delay;
	bool						ip_flag;
	bool 						ap_flag;
	bool						wifi_process;
	bool 						ap_found;
	uint8_t 					wifi_retry;
} wifi_config_t;


void ICACHE_FLASH_ATTR WiFi_Initialize(void);
void ICACHE_FLASH_ATTR WiFi_Disconnect(void);
void ICACHE_FLASH_ATTR WiFi_SetCallbacks(wifi_success_function_t, wifi_fail_function_t);
void ICACHE_FLASH_ATTR WiFi_Connect(uint32_t timeout_delay);

#endif /* USER_WIFI_H_ */
